#include <avr/io.h>
#include <util/delay.h>
#include "1wire.h"
#include "lcd.h"
#include "keyb.h"
#include "i2c.h"
#include "MCP79410.h"
#include "eeprom.h"

#define DS18B20_CONVERT_T 0x44
#define DS18B20_READ_SCRATCHPAD 0xBE

typedef struct {
	DateTime dateTime;
	float temperature;
} TemperatureRecord;

enum TemperatureState {
	TEMP_IDLE,
	TEMP_IN_PROGRESS,
	TEMP_READY
};

void getTemperature(float * temperature);
void setDateTime(void);
void displayDateTime(void);
void browseRecords(uint16_t currentRecordAddress);
void displayRecord(TemperatureRecord * temperatureRecord);

int main(void) {
	i2cInit();
	keybInit();
	lcdInit();
	lcdInitPrintf();
	//odczyt adresu ostatnio zapisanego rekordu w EEPROM
	uint16_t lastRecordAddress;
	eepromReadBytesInternal(0, (uint8_t*) &lastRecordAddress, sizeof(lastRecordAddress));
	if (lastRecordAddress > 1023 - sizeof(TemperatureRecord)) lastRecordAddress = 2;
	uint16_t browsingRecordAddress = lastRecordAddress;
	uint8_t recordWritten = 0;
	TemperatureRecord temperatureRecord;
	uint8_t browsing = 0;
	while(1){
		//odczyt temperatury
		getTemperature(&temperatureRecord.temperature);
		//odczyt daty i czasu
		DateTime * dateTime = &temperatureRecord.dateTime;
		rtcReadDateTime(dateTime);
		//zapis rekordu
		if ((dateTime->seconds % 15) == 0) {
			if (!recordWritten) {
				lastRecordAddress += sizeof(temperatureRecord);
				if (lastRecordAddress > 1023 - sizeof(temperatureRecord)) lastRecordAddress = 2;
				browsingRecordAddress = lastRecordAddress;
				eepromWriteBytesInternal(lastRecordAddress, (uint8_t *) &temperatureRecord, sizeof(temperatureRecord));
				eepromWriteBytesInternal(0, (uint8_t*) &lastRecordAddress, sizeof(lastRecordAddress));
				recordWritten = 1;
			}
		} else {
			recordWritten = 0;
		}
		//wywietlenie danych
		if (browsing) {
			TemperatureRecord historicRecord;
			eepromReadBytesInternal(browsingRecordAddress, (uint8_t *) &historicRecord, sizeof(temperatureRecord));
			displayRecord(&historicRecord);
		} else {
			displayRecord(&temperatureRecord);
		}
		//obsuga klawiatury
		if (readKeyboard()) {
			uint8_t key = getKey();
			if (browsing) {
				if (key == 1) {
					browsingRecordAddress += sizeof(temperatureRecord);
				}
				if (key == 5) {
					browsingRecordAddress -= sizeof(temperatureRecord);
				}
				if (browsingRecordAddress > 1023 - sizeof(temperatureRecord)) browsingRecordAddress = 2;
				if (key == 16) browsing = 0;
			} else {
				if (key == 1) setDateTime();
				if (key == 2) browsing = 1;
			}
		}
	}
}

void displayRecord(TemperatureRecord * temperatureRecord) {
	DateTime * dateTime = &temperatureRecord->dateTime;
	lcdGotoXY(0, 0);
	printf("%02d:%02d:%02d", dateTime->hours, dateTime->minutes, dateTime->seconds);
	lcdGotoXY(9, 0);
	printf("t=%.1f\xDF", temperatureRecord->temperature);
	lcdGotoXY(0, 1);
	printf("%02d-%02d-%4d", dateTime->dayOfMonth, dateTime->month, dateTime->year);
}



void getTemperature(float * temperature) {
	static enum TemperatureState temperatureState = TEMP_IDLE;
	switch (temperatureState) {
		case TEMP_IDLE:
			oneWireReset();
			oneWireSendByte(OW_SKIP_ROM);
			oneWireSendByte(DS18B20_CONVERT_T);
			temperatureState = TEMP_IN_PROGRESS;
			break;
		case TEMP_IN_PROGRESS:
			if (oneWireReceiveBit()) temperatureState = TEMP_READY;
			break;
		case TEMP_READY:
			oneWireReset();
			oneWireSendByte(OW_SKIP_ROM);
			oneWireSendByte(DS18B20_READ_SCRATCHPAD);
			int16_t temp;
			uint8_t * byte = (uint8_t *) &temp;
			byte[0] = oneWireReceiveByte();
			byte[1] = oneWireReceiveByte();
			*temperature = temp / 16.0;
			temperatureState = TEMP_IDLE;
			break;
	}
}

void setDateTime(void) {
	//wcz kursor
	lcdWriteCommand(LCD_COMMAND_ON_OFF | LCD_PARAM_ON_OFF_CURSOR | LCD_PARAM_ON_OFF_DISPLAY);
	//wczytaj czas
	lcdGotoXY(0, 0);
	printf("  :  :  ");
	lcdGotoXY(0, 0);
	const uint8_t timeFieldsSizes[] = {2, 2, 2};
	uint16_t time[3];
	readSeparatedNumbers(timeFieldsSizes, 3, time);
	//wczytaj dat
	lcdGotoXY(0, 1);
	printf("  -  -    ");
	lcdGotoXY(0, 1);
	const uint8_t dateFieldsSizes[] = {2, 2, 4};
	uint16_t date[3];
	readSeparatedNumbers(dateFieldsSizes, 3, date);
	DateTime newDateTime;
	newDateTime.hours = time[0];
	newDateTime.minutes = time[1];
	newDateTime.seconds = time[2];
	newDateTime.dayOfMonth = date[0];
	newDateTime.month = date[1];
	newDateTime.year = date[2];
	//zapisz nowy czas i dat
	rtcWriteDateTime(&newDateTime);
	//wycz kursor
	lcdWriteCommand(LCD_COMMAND_ON_OFF | LCD_PARAM_ON_OFF_DISPLAY);
}